/**************************************
*                                     *
*   Jeff Molofee's Basecode Example   *
*          nehe.gamedev.net           *
*                2001                 *
*                                     *
**************************************/

#include <windows.h>											// Header File For Windows
#include <gl\gl.h>												// Header File For The OpenGL32 Library
#include <gl\glu.h>												// Header File For The GLu32 Library
#include <gl\glaux.h>											// Header File For The GLaux Library
#include <stdio.h>
#include "NeHeGL.h"												// Header File For NeHeGL
#include "vectorlib.h"

#pragma comment( lib, "opengl32.lib" )							// Search For OpenGL32.lib While Linking
#pragma comment( lib, "glu32.lib" )								// Search For GLu32.lib While Linking
#pragma comment( lib, "glaux.lib" )								// Search For GLaux.lib While Linking

#ifndef CDS_FULLSCREEN											// CDS_FULLSCREEN Is Not Defined By Some
#define CDS_FULLSCREEN 4										// Compilers. By Defining It This Way,
#endif															// We Can Avoid Errors

GL_Window*	g_window;
Keys*		g_keys;

// User Defined Variables
float		angle;												// Used To Rotate The Triangles
int			rot1, rot2;											// Counter Variables

GLuint		texture[1];											// Used for Texture Mapping

AUX_RGBImageRec *LoadBMP(char *Filename)						// Loads A Bitmap Image
{
	FILE *File=NULL;											// File Handle

	if (!Filename)												// Make Sure A Filename Was Given
	{
		return NULL;											// If Not Return NULL
	}

	File=fopen(Filename,"r");									// Check To See If The File Exists

	if (File)													// Does The File Exist?
	{
		fclose(File);											// Close The Handle
		return auxDIBImageLoad(Filename);						// Load The Bitmap And Return A Pointer
	}

	return NULL;												// If Load Failed Return NULL
}

int LoadGLTextures()											// Load Bitmaps And Convert To Textures
{
	int Status=FALSE;											// Status Indicator
	AUX_RGBImageRec *TextureImage[1];							// Create Storage Space For The Texture

	memset(TextureImage, 0, sizeof(void *)*1);					// Set The Pointer To NULL

	// Load The Bitmap, Check For Errors, If Bitmap's Not Found Quit
	if (TextureImage[0]=LoadBMP("cactus.bmp"))
	{
		Status=TRUE;											// Set The Status To TRUE

		glGenTextures(1, &texture[0]);							// Create The Texture

		// Typical Texture Generation Using Data From The Bitmap
		glBindTexture(GL_TEXTURE_2D, texture[0]);

		// Generate The Texture
		glTexImage2D(GL_TEXTURE_2D, 0, 3, 
					 TextureImage[0]->sizeX, TextureImage[0]->sizeY, 
					 0, GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->data);

		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);	// Linear Filtering
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);	// Linear Filtering
	}

	if (TextureImage[0])									// If Texture Exists
	{
		if (TextureImage[0]->data)							// If Texture Image Exists
		{
			free(TextureImage[0]->data);					// Free The Texture Image Memory
		}

		free(TextureImage[0]);								// Free The Image Structure
	}

	return Status;								// Return The Status
}

BOOL GL_Window::Initialize ()									// Any GL Init Code & User Initialiazation Goes Here
{
	g_window	= this;
	g_keys		= this->keys;

	// Start Of User Initialization
	angle		= 0.0f;											// Set Starting Angle To Zero

	glClearColor (0.0f, 0.0f, 0.0f, 0.5f);						// Black Background
	glClearDepth (1.0f);										// Depth Buffer Setup
	glDepthFunc (GL_LEQUAL);									// The Type Of Depth Testing (Less Or Equal)
	glEnable (GL_DEPTH_TEST);									// Enable Depth Testing
	glShadeModel (GL_SMOOTH);									// Select Smooth Shading
	glHint (GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);			// Set Perspective Calculations To Most Accurate

	if (!LoadGLTextures())										// Jump To Texture Loading Routine
	{
		return FALSE;											// If Texture Didn't Load Return FALSE
	}
	glEnable(GL_TEXTURE_2D);									// Enable Texture Mapping ( NEW )

	return TRUE;												// Return TRUE (Initialization Successful)
}

void GL_Window::Deinitialize (void)								// Any User DeInitialization Goes Here
{
}

void Update (DWORD milliseconds)								// Perform Motion Updates Here
{
	if (g_keys->keyDown [VK_ESCAPE] == TRUE)					// Is ESC Being Pressed?
	{
		g_window->TerminateApplication ();						// Terminate The Program
	}

	if (g_keys->keyDown [VK_F1] == TRUE)						// Is F1 Being Pressed?
	{
		g_window->ToggleFullscreen ();							// Toggle Fullscreen Mode
	}

	angle += (float)(milliseconds) / 5.0f;						// Update angle Based On The Clock
}

void GL_Window::Draw (void)
{
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);		// Clear Screen And Depth Buffer
	glLoadIdentity ();											// Reset The Modelview Matrix
	glTranslatef (0.0f, 0.0f, -6.0f);							// Translate 6 Units Into The Screen
	glRotatef (angle, 0.0f, 1.0f, 0.0f);						// Rotate On The Y-Axis By angle

	// * 𵨺  ´ *
	float mat[16];
	glGetFloatv(GL_MODELVIEW_MATRIX, mat);

	// * 𵨺 ķκ ü⿡    Ϳ   ͸ Ѵ.
	vector3_t right(mat[0], mat[4], mat[8]);
	vector3_t up(mat[1], mat[5], mat[9]);

	glBindTexture(GL_TEXTURE_2D, texture[0]);					// Select Our Texture

	vector3_t pos(0.0f, 0.0f, 1.0f);

	glBegin(GL_QUADS);
//		glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);	// Bottom Left Of The Texture and Quad
//		glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);	// Bottom Right Of The Texture and Quad
//		glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f,  1.0f);	// Top Right Of The Texture and Quad
//		glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f,  1.0f);	// Top Left Of The Texture and Quad
		glTexCoord2f(0.0f, 0.0f); glVertex3fv((pos+(right+up)*(-1)).v);	// Bottom Left Of The Texture and Quad
		glTexCoord2f(1.0f, 0.0f); glVertex3fv((pos+(right-up)*(+1)).v);	// Bottom Right Of The Texture and Quad
		glTexCoord2f(1.0f, 1.0f); glVertex3fv((pos+(right+up)*(+1)).v);	// Top Right Of The Texture and Quad
		glTexCoord2f(0.0f, 1.0f); glVertex3fv((pos+(up-right)*(+1)).v);	// Top Left Of The Texture and Quad
	glEnd();

	glFlush ();													// Flush The GL Rendering Pipeline
}
